<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Client;

use Psr\Http\Message\ResponseInterface;

class Client extends \GuzzleHttp\Client
{
    public const SUCCESS = 0;
    public const INITIALIZING = 1;
    public const INTERNAL_ERROR = 2;
    public const INVALID_REQUEST = 3;
    public const INVALID_PARAM = 4;
    public const GUARDED = 5;
    public const TIMEOUT = 6;
    public const FIRMWARE_UPDATING = 99;
    public const ACCESS_ERROR = 100;
    public const OTHER_ERRORS = 101;
    public const WRONG_USER_NAME = 102;
    public const WRONG_PASSWORD = 103;
    public const ACCOUNT_EXPIRED = 104;
    public const ACCOUNT_DISCONNECTED = 105;
    public const ACCOUNT_REACHED_LIMIT = 106;
    public const SERVER_MAINTENANCE = 107;
    public const INVALID_ACCOUNT = 108;
    public const LICENSE_ERROR = 109;
    public const READ_ONLY_MODE = 110;
    public const MAX_STATIONS = 111;
    public const ACCESS_DENIED = 112;
    public const UNKNOWN_ERROR = 36200;
    public const INVALID_RECEIVER_INPUT = 36201;

    public const RESPONSE_MAP = [
      self::SUCCESS => 'Successful request',
      self::INITIALIZING => 'Initializing',
      self::INTERNAL_ERROR => 'Internal Error',
      self::INVALID_REQUEST => 'Invalid Request (A method does not exist, or was not appropriate.)',
      self::INVALID_PARAM => 'Invalid Parameter (Out of range, invalid characters etc.)',
      self::GUARDED => 'Guarded (Unable to setup in current status.)',
      self::TIMEOUT => 'Time Out',
      self::FIRMWARE_UPDATING => 'Firmware Updating',
      self::ACCESS_ERROR => 'Access Error',
      self::OTHER_ERRORS => 'Other Errors',
      self::WRONG_USER_NAME => 'Wrong User Name',
      self::WRONG_PASSWORD => 'Wrong Password',
      self::ACCOUNT_EXPIRED => 'Account Expired',
      self::ACCOUNT_DISCONNECTED => 'Account Disconnected / Gone Off / Shut Down',
      self::ACCOUNT_REACHED_LIMIT => 'Account Number Reached to the Limit',
      self::SERVER_MAINTENANCE => 'Server Maintenance',
      self::INVALID_ACCOUNT => 'Invalid Account',
      self::LICENSE_ERROR => 'License Error',
      self::READ_ONLY_MODE => 'Read Only Mode',
      self::MAX_STATIONS => 'Max Stations',
      self::ACCESS_DENIED => 'Access Denied',
      self::UNKNOWN_ERROR => 'Unknown Error',
      self::INVALID_RECEIVER_INPUT => 'Input channel does not exist.',
    ];

    public const VOLUME_DIRECTION_UP = 'up';
    public const VOLUME_DIRECTION_DOWN = 'down';
    private const APIBASE = '/YamahaExtendedControl/v2/';

    private $cacheTimeout;

    public function __construct(string $receiverBaseUri, $cacheTimeout = null)
    {
        $this->cacheTimeout = $cacheTimeout;

        if (null === $cacheTimeout) {
            $this->cacheTimeout = new \DateTime('now +1 hour');
        }

        $baseUri = sprintf('%s', rtrim($receiverBaseUri, '/'));
        $baseUri .= self::APIBASE;

        if (false === strpos($baseUri, '://')) {
            $baseUri = sprintf('http://%s', $baseUri);
        }

        parent::__construct([
          'base_uri' => $baseUri,
          'http_errors' => false,
        ]);
    }

    public function boolToStr(bool $value): string
    {
        return $value ? 'true' : 'false';
    }

    public function getCacheTimeOut(): \DateTime
    {
        return $this->cacheTimeout;
    }

    protected function checkResponse(ResponseInterface $response): bool
    {
        return 200 === $response->getStatusCode();
    }

    private function floatToStr(float $value): string
    {
        return (string)$value;
    }

    private function intToStr(int $value): string
    {
        return (string)$value;
    }
}
