<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Client;

use Psr\Http\Message\ResponseInterface;

class System
{
    private $client;

    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    public function getFeatures(): array
    {
        $cache = null;

        if (file_exists(sys_get_temp_dir() . '/yamaha-av-features.array')) {
            $cache = unserialize(base64_decode(file_get_contents(sys_get_temp_dir() .'/yamaha-av-features.array'), true), [__CLASS__]);
        }

        if (null !== $cache && $this->client->getCacheTimeOut() < $cache['date']) {
            print 'Returning Cache';
            return $cache['features'];
        }

        $features = $this->client->get('system/getFeatures');

        if ($this->checkResponse($features)) {
            $features = \json_decode($features->getBody()->getContents(), true);
            $cache['date'] = new \DateTime('now');
            $cache['features'] = $features;
            file_put_contents(sys_get_temp_dir() . '/yamaha-av-features.array', base64_encode(serialize($cache)));
            return $features;
        }

        return [];
    }

    protected function checkResponse(ResponseInterface $response): bool
    {
        return 200 === $response->getStatusCode();
    }
}
