<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use Doctrine\DBAL\Connection;
use Sikofitt\Yamaha\AV\Repository\UserRepository;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Creates an api token for a user.
 *
 * {@inheritDoc}
 */
class CreateApiTokenCommand extends Command
{
    /**
     * @var UserRepository
     */
    private $userRepository;

    /**
     * CreateApiTokenCommand constructor.
     *
     * @param \Sikofitt\Yamaha\AV\Repository\UserRepository $userRepository
     *
     * @throws \Symfony\Component\Console\Exception\LogicException
     */
    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;

        parent::__construct();
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('user:token')
            ->setDescription('Creates an api token for the given user.')
            ->addArgument('username', InputArgument::REQUIRED, 'The user to create the token for.');
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        $io = new SymfonyStyle($input, $output);

        $userName = $input->getArgument('username');

        if (false === $this->userRepository->createApiTokenForUser($userName)) {
            $io->error('Failed to created token for user ' . $userName . '.  Are you sure the user exists?');
            return 1;
        }

        $io->success('Created token for user ' . $userName . '.');

        return 0;
    }
}
