<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use Sikofitt\Yamaha\AV\Client\Client;
use Sikofitt\Yamaha\AV\Client\Zone;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\LogicException;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class InputCommand extends Command
{
    /**
     * Configures the current command.
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('input')
            ->setDescription('Sets the receiver input to the specified input.'.PHP_EOL.'Use \'list\' to get a list of available inputs.')
            ->addArgument('input', InputArgument::REQUIRED, 'Set the input on the receiver.');
    }

    /**
     * Executes the current command.
     *
     * This method is not abstract because you can use this class
     * as a concrete class. In this case, instead of defining the
     * execute() method, you set the code to execute by passing
     * a Closure to the setCode() method.
     *
     * @param InputInterface $input An InputInterface instance
     * @param OutputInterface $output An OutputInterface instance
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @throws LogicException When this abstract method is not implemented
     * @return null|int null or 0 if everything went fine, or an error code
     *
     *
     * @see setCode()
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        $receiverInput = strtolower($input->getArgument('input'));
        $client = new Client('10.5.7.230');
        $zoneClient = new Zone($client, $input->getOption('zone'));
        if ('list' === $receiverInput) {
            $inputChannels = $zoneClient->getInputsForZone($input->getOption('zone'));
            $output->writeln(sprintf('Available channels are : %s', implode(', ', $inputChannels)));
            return 0;
        }

        $result = $zoneClient->setInput($input->getArgument('input'));
        $output->writeln(Client::RESPONSE_MAP[$result]);
        return $result;
    }
}
