<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use PHP_CodeSniffer\Tests\Core\File\testFECNNonExtendedClass;
use Sikofitt\Yamaha\AV\Client\Client;
use Sikofitt\Yamaha\AV\Client\Zone;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\LogicException;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

class MuteCommand extends Command
{
    /**
     * Configures the current command.
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('mute')
            ->setDescription('Mute the AV Receiver.')
            ->addArgument('uri', InputArgument::REQUIRED, 'Yamaha Receiver URI')
            ->addOption('disable', 'd', InputOption::VALUE_NONE, 'disable mute.');
    }

    /**
     * Executes the current command.
     *
     * This method is not abstract because you can use this class
     * as a concrete class. In this case, instead of defining the
     * execute() method, you set the code to execute by passing
     * a Closure to the setCode() method.
     *
     * @param InputInterface $input An InputInterface instance
     * @param OutputInterface $output An OutputInterface instance
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @throws LogicException When this abstract method is not implemented
     * @return null|int null or 0 if everything went fine, or an error code
     *
     *
     * @see setCode()
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        try {
            $zoneOption = strtolower($input->getOption('zone'));
        } catch (\Exception $e) {
            $zoneOption = 'main';
        }

        $avClient = new Client($input->getArgument('uri'));
        $client = new Zone($avClient, $input->getOption('zone'));
        $status = $client->getStatus();
        if (isset($status['mute'])) {
            $commandResult = $client->setMute(!$status['mute']);
            if (isset(Client::RESPONSE_MAP[$commandResult])) {
                $output->writeln(Client::RESPONSE_MAP[$commandResult]);
                return $commandResult;
            }
        }

        return Client::UNKNOWN_ERROR;
    }
}
