<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use Doctrine\DBAL\Connection;
use Sikofitt\Yamaha\AV\Client\Client;
use Sikofitt\Yamaha\AV\Client\System;
use Sikofitt\Yamaha\AV\Client\Zone;
use Sikofitt\Yamaha\AV\Repository\UserRepository;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class StatusCommand extends Command
{
    private $uri;
    private $connection;

    public function __construct(Connection $connection)
    {
        $this->connection = $connection;

        parent::__construct();
    }

    public function configure(): void
    {
        $this->setName('status')
        ->setDescription('Get current status information.')
        ->addArgument('uri', InputArgument::REQUIRED, 'Yamaha AV Receiver Host or IP Address.');
    }

    public function interact(InputInterface $input, OutputInterface $output): void
    {
        $this->uri = strtolower($input->getArgument('uri'));
        if (false === strpos($this->uri, '://')) {
            $this->uri = sprintf('http://%s', $this->uri);
        }
    }

    /**
     * @param \Symfony\Component\Console\Input\InputInterface $input
     * @param \Symfony\Component\Console\Output\OutputInterface $output
     * 10.5.7.230
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     *
     * @return int|null
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        $io = new SymfonyStyle($input, $output);
        $client = new Client($this->uri);
        $zoneClient = new Zone($client, $input->getOption('zone'));
        $systemClient = new System($client);
        $status = $zoneClient->getStatus();
        $mainZone = $zoneClient->getInputsForZone();
        $features = $systemClient->getFeatures();
        //$io->listing($status);
        $io->title('Zone Inputs');
        $io->block($mainZone);
        //$io->listing($features);
        //dump($status);
        //dump($mainZone);
        $repo = new UserRepository($this->connection);
        dump($repo->getApiToken('sikofitt'));
        return 0;
    }
}
