<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use Sikofitt\Yamaha\AV\Client\Client;
use Sikofitt\Yamaha\AV\Client\Zone;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

class VolumeCommand extends Command
{
    public function configure(): void
    {
        $this->setName('volume')
            ->setDescription('Control the volume')
            ->addArgument('step', InputArgument::OPTIONAL, 'How much to increase or decrease the volume', 0.5)
            ->addOption('up', 'u', InputOption::VALUE_NONE, 'Turn the volume up.')
            ->addOption('down', 'd', InputOption::VALUE_NONE, 'Turn the volume down');
    }

    public function execute(InputInterface $input, OutputInterface $output)
    {
        $up = $input->getOption('up');
        $down = $input->getOption('down');
        $step = $input->getArgument('step');

        $avClient = new Client('10.5.7.230');
        $client = new Zone($avClient, $input->getOption('zone'));

        if (false === $up && false === $down) {
            $percent = (bool)strpos($step, '%');

            if ($percent) {
                $result = $client->changeVolumeByPercent((float)$step);
            } else {
                $result = $client->changeVolume((int)$step);
            }

            $output->writeln(Client::RESPONSE_MAP[$result]);

            return $result;
        }

        if (true === $up && true === $down) {
            $output->writeln('Can only choose up or down.');
            exit;
        }

        if (true === $up) {
            $result = $client->increaseVolume($step);
            $output->writeln(Client::RESPONSE_MAP[$result]);
            return $result;
        }

        if (true === $down) {
            $result = $client->decreaseVolume($step);
            $output->writeln(Client::RESPONSE_MAP[$result]);
            return $result;
        }
    }
}
